@@ -1,7 +1,8 @@
 /*
  * group-mixin.c - Source for TpGroupMixin
- * Copyright (C) 2006 Collabora Ltd.
- * Copyright (C) 2006 Nokia Corporation
+ *
+ * Copyright (C) 2006-2007 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright (C) 2006-2007 Nokia Corporation
  *   @author Ole Andre Vadla Ravnaas <ole.andre.ravnaas@collabora.co.uk>
  *   @author Robert McQueen <robert.mcqueen@collabora.co.uk>
  *
@@ -41,8 +42,8 @@
  * tp_group_mixin_iface_init)</literal> in the fourth argument to
  * <literal>G_DEFINE_TYPE_WITH_CODE</literal>.
  *
- * You can also implement the group interface by forwarding all group
- * operations to the group mixin of an associated object (mainly useful
+ * Since 0.5.13 you can also implement the group interface by forwarding all
+ * group operations to the group mixin of an associated object (mainly useful
  * for Tubes channels). To do this, call tp_external_group_mixin_init()
  * in the constructor after the associated object has been set,
  * tp_external_group_mixin_finalize() in the dispose or finalize function, and
@@ -58,18 +59,14 @@
 
 #include <telepathy-glib/debug-ansi.h>
 #include <telepathy-glib/errors.h>
+#include <telepathy-glib/gtypes.h>
 
 #define DEBUG_FLAG TP_DEBUG_GROUPS
 
-#include "internal-debug.h"
+#include "debug-internal.h"
 
 #include "_gen/signals-marshal.h"
 
-#define TP_CHANNEL_GROUP_LOCAL_PENDING_WITH_INFO_ENTRY_TYPE \
-    (dbus_g_type_get_struct ("GValueArray", \
-        G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING, \
-        G_TYPE_INVALID))
-
 static const char *
 group_change_reason_str (guint reason)
 {
@@ -181,6 +178,8 @@ tp_group_mixin_get_offset_quark ()
  * initialization, the given callback will be used instead of the remove
  * callback passed to tp_group_mixin_class_init() (which must be %NULL
  * in this case).
+ *
+ * Since: 0.5.13
  */
 void
 tp_group_mixin_class_set_remove_with_reason_func (GObjectClass *cls,
@@ -689,7 +688,7 @@ tp_group_mixin_remove_members_async (TpSvcChannelInterfaceGroup *obj,
 /**
  * tp_group_mixin_get_members:
  * @obj: An object implementing the group interface using this mixin
- * @ret: Used to return a GArray of guint contact handles
+ * @ret: Used to return a newly-allocated GArray of guint contact handles
  * @error: Unused
  *
  * Get the group's current members
@@ -731,7 +730,7 @@ tp_group_mixin_get_members_async (TpSvcChannelInterfaceGroup *obj,
 /**
  * tp_group_mixin_get_local_pending_members:
  * @obj: An object implementing the group interface using this mixin
- * @ret: Used to return a GArray of guint contact handles
+ * @ret: Used to return a newly-allocated GArray of guint contact handles
  * @error: Unused
  *
  * Get the group's local-pending members.
@@ -782,15 +781,14 @@ local_pending_members_with_info_foreach (TpHandleSet *set,
 {
   _mixin_and_array_of_info *data = (_mixin_and_array_of_info *)userdata;
   TpGroupMixinPrivate *priv = data->mixin->priv;
+  GType info_type = TP_STRUCT_TYPE_LOCAL_PENDING_INFO;
   GValue entry = { 0, };
   LocalPendingInfo *info = g_hash_table_lookup (priv->local_pending_info,
                                                 GUINT_TO_POINTER(i));
   g_assert (info != NULL);
 
-  g_value_init (&entry, TP_CHANNEL_GROUP_LOCAL_PENDING_WITH_INFO_ENTRY_TYPE);
-  g_value_take_boxed (&entry,
-      dbus_g_type_specialized_construct (
-          TP_CHANNEL_GROUP_LOCAL_PENDING_WITH_INFO_ENTRY_TYPE));
+  g_value_init (&entry, info_type);
+  g_value_take_boxed (&entry, dbus_g_type_specialized_construct (info_type));
 
   dbus_g_type_struct_set (&entry,
       0, i,
@@ -805,8 +803,8 @@ local_pending_members_with_info_foreach (TpHandleSet *set,
 /**
  * tp_group_mixin_get_local_pending_members_with_info:
  * @obj: An object implementing the group interface using this mixin
- * @ret: Used to return a GPtrArray of D-Bus structures each containing
- *  the handle of a local-pending contact, the handle of a contact
+ * @ret: Used to return a newly-allocated GPtrArray of D-Bus structures each
+ * containing the handle of a local-pending contact, the handle of a contact
  *  responsible for adding them to the group (or 0), the reason code
  *  and a related message (e.g. their request to join the group)
  * @error: Unused
@@ -863,8 +861,8 @@ tp_group_mixin_get_local_pending_members_with_info_async (
 /**
  * tp_group_mixin_get_remote_pending_members:
  * @obj: An object implementing the group interface using this mixin
- * @ret: Used to return a GArray of guint representing the handles
- *  of the group's remote pending members
+ * @ret: Used to return a newly-allocated GArray of guint representing the
+ * handles of the group's remote pending members
  * @error: Unused
  *
  * Get the group's remote-pending members.
@@ -906,12 +904,12 @@ tp_group_mixin_get_remote_pending_members_async (TpSvcChannelInterfaceGroup *obj
 /**
  * tp_group_mixin_get_all_members:
  * @obj: An object implementing the group interface using this mixin
- * @members: Used to return a GArray of guint representing the handles
- *  of the group's members
- * @local_pending: Used to return a GArray of guint representing the handles
- *  of the group's local pending members
- * @remote_pending: Used to return a GArray of guint representing the handles
- *  of the group's remote pending members
+ * @members: Used to return a newly-allocated GArray of guint representing
+ * the handles of the group's members
+ * @local_pending: Used to return a newly-allocated GArray of guint
+ * representing the handles of the group's local pending members
+ * @remote_pending: Used to return a newly-allocated GArray of guint
+ * representing the handles of the group's remote pending members
  * @error: Unused
  *
  * Get the group's current and pending members.
@@ -1227,7 +1225,8 @@ local_pending_remove (TpGroupMixin *mixin,
 /**
  * tp_group_mixin_change_members:
  * @obj: An object implementing the group interface using this mixin
- * @message: A message to be sent to the affected contacts if possible
+ * @message: A message to be sent to the affected contacts if possible;
+ *  %NULL is allowed, and is mapped to an empty string
  * @add: A set of contact handles to be added to the members (if not
  *  already present) and removed from local pending and remote pending
  *  (if present)
@@ -1250,6 +1249,9 @@ local_pending_remove (TpGroupMixin *mixin,
  * If any two of add, remove, add_local_pending and add_remote_pending have
  * a non-empty intersection, the result is undefined. Don't do that.
  *
+ * Each of the TpIntSet arguments may be %NULL, which is treated as
+ * equivalent to an empty set.
+ *
  * Returns: %TRUE if the group was changed and the MembersChanged signal
  *  was emitted; %FALSE if nothing actually changed and the signal was
  *  suppressed.
@@ -1271,6 +1273,9 @@ tp_group_mixin_change_members (GObject *obj,
 
   empty = tp_intset_new ();
 
+  if (message == NULL)
+    message = "";
+
   if (add == NULL)
     add = empty;
 
@@ -1546,6 +1551,8 @@ _external_group_mixin_get_obj_quark (void)
  * Fill in the qdata needed to implement the group interface using
  * the group mixin of another object. This function should usually be called
  * in the instance constructor.
+ *
+ * Since: 0.5.13
  */
 void
 tp_external_group_mixin_init (GObject *obj, GObject *obj_with_mixin)
@@ -1563,6 +1570,8 @@ tp_external_group_mixin_init (GObject *obj, GObject *obj_with_mixin)
  *
  * Remove the external group mixin. This function should usually be called
  * in the dispose or finalize function.
+ *
+ * Since: 0.5.13
  */
 void
 tp_external_group_mixin_finalize (GObject *obj)
@@ -1705,6 +1714,8 @@ tp_external_group_mixin_remove_members_with_reason_async
  * Fill in the vtable entries needed to implement the group interface using
  * the group mixin of another object. This function should usually be called
  * via G_IMPLEMENT_INTERFACE.
+ *
+ * Since: 0.5.13
  */
 void
 tp_external_group_mixin_iface_init (gpointer g_iface,
