@@ -47,6 +47,8 @@
  * G_IMPLEMENT_INTERFACE, you should call tp_presence_mixin_iface_init.
  * TpPresenceMixin implements all of the D-Bus methods in the Presence
  * interface.
+ *
+ * Since: 0.5.13
  */
 
 #include <telepathy-glib/presence-mixin.h>
@@ -57,10 +59,11 @@
 #include <telepathy-glib/base-connection.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/errors.h>
+#include <telepathy-glib/gtypes.h>
 
 #define DEBUG_FLAG TP_DEBUG_PRESENCE
 
-#include "internal-debug.h"
+#include "debug-internal.h"
 
 
 struct _TpPresenceMixinPrivate
@@ -82,8 +85,7 @@ deep_copy_hashtable (GHashTable *hash_table)
   if (!hash_table)
     return NULL;
 
-  g_value_init (&value,
-      dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE));
+  g_value_init (&value, TP_HASH_TYPE_STRING_VARIANT_MAP);
   g_value_take_boxed (&value, hash_table);
   return g_value_dup_boxed (&value);
 }
@@ -173,7 +175,8 @@ tp_presence_mixin_get_offset_quark ()
  * @offset: The byte offset of the TpPresenceMixinClass within the class
  * structure
  * @status_available: A callback to be used to determine if a given presence
- *  status is available. If NULL, all statuses are always considered available.
+ *  status is available to be set on the connection. If NULL, all statuses are
+ *  always considered available.
  * @get_contact_statuses: A callback to be used get the current presence status
  *  for contacts. This is used in implementations of various D-Bus methods and
  *  hence must be provided.
@@ -315,8 +318,7 @@ construct_presence_hash_foreach (gpointer key,
 
   g_value_array_append (vals, NULL);
   g_value_init (g_value_array_get_nth (vals, 1),
-      dbus_g_type_get_map ("GHashTable", G_TYPE_STRING,
-        dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)));
+      TP_HASH_TYPE_MULTIPLE_STATUS_MAP);
   g_value_take_boxed (g_value_array_get_nth (vals, 1), contact_status);
 
   g_hash_table_insert (data->presence_hash, GUINT_TO_POINTER (handle), vals);
@@ -625,7 +627,7 @@ tp_presence_mixin_remove_status (TpSvcConnectionInterfacePresence *iface,
   GObject *obj = (GObject *) iface;
   TpBaseConnection *conn = TP_BASE_CONNECTION (iface);
   TpPresenceMixinClass *mixin_cls =
-    TP_PRESENCE_MIXIN_CLASS (G_OBJECT_CLASS (iface));
+    TP_PRESENCE_MIXIN_CLASS (G_OBJECT_GET_CLASS (obj));
   GArray *self_contacts;
   GError *error = NULL;
   GHashTable *self_contact_statuses;
